/******************************************************************************
 *
 *  Project:        Daisy Client Library
 *
 *  Filename:       ecc_helpers.h
 *
 *  Purpose:        Control Library for ECC100
 *
 *  Author:         NHands GmbH & Co KG
 */
/*****************************************************************************/
/** @file ecc_helpers.h
 *  @brief Control Library for ECC100
 *
 *  ecc_helpers is a library for the control of the ECC100 piezo controller.
 *  To run properly, the additional library "daisybase" is required and a
 *  daisy installation must be available.
 */
/*****************************************************************************/
/** @mainpage ECC Helpers Lib for Daisy
 *
 *  \ref ecc_helpers.h "Ecc_helpers" is a library that allows custom
 *  programming for the ECC100 piezo controller. It is related to "Daisy",
 *  the control and data acquisition software for ECC.
 *  ecc_helpers based programs start Daisy's application server in background,
 *  so that Daisy and the custom program can be used in parallel.
 *
 *  Documentation for ecc_helper's convenience functions can be found
 *  \ref ecc_helpers.h "here".
 */
/*****************************************************************************/
/* $Id: ecc_helpers.h,v 1.7 2012/04/13 10:51:12 trurl Exp $ */

#ifndef __ECC_HELPERS_H__
#define __ECC_HELPERS_H__

#ifdef __cplusplus
#define EXTC extern "C"                     /**< For use with C++            */
#else
#define EXTC extern                         /**< For use with C              */
#endif
#ifdef unix
#define ECC_API EXTC                        /**< Not required for Unix       */
#define ECC_CC                              /**< Not required for Unix       */
#else
#define ECC_CC       __stdcall              /**< Calling convention          */
#ifdef  ECC_EXPORTS
#define ECC_API EXTC __declspec(dllexport)  /**< For internal use            */
#else
#define ECC_API EXTC __declspec(dllimport)  /**< For external use            */
#endif
#endif

typedef int    Int32;                       /**< 32 bit integer              */
typedef int    Bln32;                       /**< Boolean (for documentation) */

  
/**  @brief Return codes of the functions                                    */
typedef enum {
  ECC_Ok,                               /**< No error                        */
  ECC_Error,                            /**< Unknown / other error           */
  ECC_Timeout,                          /**< Communication timeout           */
  ECC_NotConnected,                     /**< No contact to controller via USB*/
  ECC_DriverError,                      /**< Error when calling USB driver   */
  ECC_FileNotFound,                     /**< Controller boot image not found */
  ECC_SrvNotFound,                      /**< Server executable not found     */
  ECC_ServerLost,                       /**< No contact to the server        */
  ECC_OutOfRange,                       /**< Invalid parameter in fct call   */
  ECC_WrongContext,                     /**< Call in invalid thread context  */
  ECC_XmlError,                         /**< Invalid format of profile file  */
  ECC_OpenError                         /**< Can't open specified file       */
} ECC_Rc;


/** @brief  Actor types                                                      */
typedef enum {
  ECC_actorLinear,                      /**< Actor is of linear type         */
  ECC_actorGonio,                       /**< Actor is of goniometer type     */
  ECC_actorRot                          /**< Actor is of rotator type        */
} ECC_actorType;



/** @brief Initialize data
 *
 *  Initializes internal data. An application server is
 *  started (if not already running) and connected.
 *
 *  @param   binPath Path to daisy binaries, in particular daisysrv(.exe)
 *  @return          0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_setInit( const char * binPath );


/** @brief Uninitialize
 *
 *  Stops the internal event loop and shuts down the application server
 *  connection.
 *
 *  @return          0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_setDeInit();


/** @brief Control output stage
 *
 *  Controls the output relais of the selected axis.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  enable     Switches the output relais
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlOutput( Int32 axis,
                                         Bln32* enable,
                                         Bln32 set );


/** @brief Control amplitude
 *
 *  Controls the amplitude of the actuator signal.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  amplitude  Amplitude in mV
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlAmplitude( Int32 axis,
                                            Int32* amplitude,
                                            Bln32 set );


/** @brief Control frequency
 *
 *  Controls the frequency of the actuator signal.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  frequency  Frequency in mHz
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlFrequency( Int32 axis,
                                            Int32* frequency,
                                            Bln32 set );



/** @brief Control actor selection
 *
 *  Selects the actor to be used on selected axis from actor presets
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  actor      Actor selection [0..255]
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlActorSelection( Int32 axis,
                                                 Int32* actor,
                                                 Bln32 set );


/** @brief Get actor name
 *
 *  Get the name of actual selected actor
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  name       Name of the actor coded as NULL-terminated C-string.
 *                     User must provide at least 20 bytes of allocated memory.
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getActorName( Int32 axis,
                                 char* name );


/** @brief Get actor type
 *
 *  Get the type of actual selected actor
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  type       Type of the actor. See @ref ECC_actorType
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getActorType( Int32 axis,
                                        ECC_actorType* type );


/** @brief Save params
 *
 *  Saves user parameters to persistant flash in controller.
 *  Parameters that will be saved are amplitude, 
 *  frequency and actor selection of each axis.
 *
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_setSaveParams( void );


/** @brief Writing flash status
 *
 *  Retrieves the status of writing to flash. Indicates whether a write access
 *  to the flash is performing.
 *
 *  @param  writing    status
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getStatusFlash( Bln32* writing );



/** @brief Reset position
 *
 *  Resets the actual position to zero and marks the reference position as invalid.
 *
 *  @param  axis       Number of the axis to be configured
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_setReset( Int32 axis );


/** @brief Control actor approach
 *
 *  Controls the approach of the actor to the target position
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  enable     Enables/ disables the approach
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlMove( Int32 axis,
                                       Bln32* enable,
                                       Bln32 set );



/** @brief Single step
 *
 *  Triggers a single step in desired direction.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  backward   Selects the desired direction. False triggers a forward step,
 *                     true a backward step.
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_setSingleStep( Int32 axis,
                                         Bln32 backward );



/** @brief Control continous movement forward
 *
 *  Controls continous movement in forward direction. If enabled a potential present movement
 *  in the opposite direction is stopped. The parameter "enabled" set to false stops all
 *  movement of the axis regardless of its direction.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  enable     Enables/ disables the movement
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlContinousFwd( Int32 axis,
                                               Bln32* enable,
                                               Bln32 set );



/** @brief Control continous movement backward
 *
 *  Controls continous movement in backward direction. If enabled a potential present movement
 *  in the opposite direction is stopped. The parameter "enable" set to false stops all movement
 *  of the axis regardless of its direction.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  enable     Enables/ disables the movement
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlContinousBkwd( Int32 axis,
                                                Bln32* enable,
                                                Bln32 set );



/** @brief Control target position
 *
 *  Controls the target position for the approach function, see @ref ECC_controlMove .
 *  For linear type actors the position is defined in pm for goniometer an rotator type
 *  actors it is .
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  target     Target position in pm or  depending on actor type.
 *  @param  set        1: Send the supplied values to the controller
 *                     0: Ignore input; only retreive the results
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_controlTargetPosition( Int32 axis,
                                                 Int32* target,
                                                 Bln32 set );



/** @brief Reference status
 *
 *  Retrieves the status of the reference position. It may be valid or invalid.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  valid      Status of the reference position.
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getStatusReference( Int32 axis,
                                              Bln32* valid );



/** @brief Moving status
 *
 *  Retrieves the moving status. Moving means the actor is actively driven by the output stage
 *  either for approaching or continous/single stepping.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  moving     Moving status
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getStatusMoving( Int32 axis,
                                           Bln32* moving );


/** @brief Error status
 *
 *  Retrieves the error status. Indicates a sensor malfunction.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  error      Error status. "True" means the occurrence of an error.
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getStatusError( Int32 axis,
                                          Bln32* error );


/** @brief Connected status
 *
 *  Retrieves the connected status. Indicates whether an actor is eletrically connected
 *  to the controller.
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  connected  status
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getStatusConnected( Int32 axis,
                                              Bln32* connected );


/** @brief Reference position
 *
 *  Retrieves the reference position. See @ref ECC_getStatusReference for 
 *  determining the validity.
 *  For linear type actors the position is defined in pm for goniometer an rotator type
 *  actors it is .
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  reference  Reference position in pm or  depending on actor type.
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getReferencePosition( Int32 axis,
                                                Int32* reference );




/** @brief Actor position
 *
 *  Retrieves the actual actor position.
 *  For linear type actors the position is defined in pm for goniometer an rotator type
 *  actors it is .
 *
 *  @param  axis       Number of the axis to be configured
 *  @param  position   Reference position in pm or  depending on actor type.
 *  @return            0: Success  !=0: Error
 */
ECC_API ECC_Rc ECC_CC ECC_getPosition( Int32 axis,
                                       Int32* position );


#endif
